"use strict";
const CryptoJS = require("crypto-js");
const http = require("http");
var fileupload = require("express-fileupload");
var multer  = require('multer')


var storage = multer.diskStorage(
    {
        destination: './files/',
        filename: function ( req, file, cb ) {
            cb( null, Math.round(Math.random()*100) + Date.now().toString() );
        }
    }
);
var upload = multer( { storage: storage } );


const express = require('express');
const app = express();
const port = 8080;
const sql = require('./sql-request');


///// JWT part /////

var valid_tokens = [];

String.prototype.hashCode = function() {
    var hash = 0, i, chr;
    if (this.length === 0) return hash;
    for (i = 0; i < this.length; i++) {
        chr   = this.charCodeAt(i);
        hash  = ((hash << 5) - hash) + chr;
        hash |= 0; // Convert to 32bit integer
    }
    return hash;
};

function tob64(elem){
    return Buffer.from(elem.toString()).toString('base64').slice(0, -2);
}

function Header(typ, alg){
    this.typ = typ;
    this.alg = alg;

    this.toString = function(){ return "{" + this.typ + "," + this.alg + "}"};
}

function Payload(user, pass_enc){
    this.user = user;
    this.pass_enc = pass_enc;

    this.toString = function(){ return "{" + this.user + "," + this.pass_enc + "}"};

}

function Signature(token){
    const secret = "our super hyperdrive secret" + new Date();
    return CryptoJS.HmacSHA512(token, secret);
}

function JWT(pl_user, pl_pass){
    this.header = new Header("jwt", "HS512");
    this.payload = new Payload(pl_user, pl_pass.hashCode());
    let token = tob64(this.header) + "." + tob64(this.payload);
    let signature = new Signature("token");
    this.signedToken = token + "." + signature;
}


function add_token(token, user) {

    for (let i = 0; i < valid_tokens.length; i++) {
        if(Object.keys(valid_tokens[i])[0] == token){
            valid_tokens.splice(i, 1);
            console.log("Unable to add token. (token already present)");
            break;
        }
    }
    let obj = {}; obj[token.toString()] = user;
    valid_tokens.push(obj);
    console.log(valid_tokens)

    console.log("Successfully added token.");

    
}

function remove_token(token) {

    for (let i = 0; i < valid_tokens.length; i++) {
        if(Object.keys(valid_tokens[i])[0] == token){
            valid_tokens.splice(i, 1);
            console.log("Successfully removed token from valid_tokens.");
            return true;
        }
    }

    console.log("Unable to remove token from valid_tokens. (Token not present)");


}

// verify the token
// return : user if exist
function verify_token(token) {
    for (let i = 0; i < valid_tokens.length; i++) {
        if(Object.keys(valid_tokens[i])[0] == token)
            return valid_tokens[i][token];
    }
    return false;
}


///// End JWT part /////


app.get('/', (req, res) => {
    res.sendFile(__dirname + '/front/index.html');
});

/* Login
 * param : pseudo
 * param : password
 */
// resCode : [ 0: User now logged in, 1: False password, 2: Invalid username, 3: Empty user or pass ]
app.get('/login', (req, res) => {

    const user = req.query['user'];
    const pass = req.query['pass'];
    sql.userExist(user, pass, (element) => {
        if (element)
            check_login(user, pass, element, res);
    });

});

function check_login(user, pass, userObject, res) {
    if (!user || !pass) {
        res.send({
            "route": "/login",
            "resCode": 4,
            "comment": "Please enter a username and a password."
        })
    }
    else{
        if (userObject){
            if (userObject.passwd == pass.hashCode()) {
                let jwt = new JWT(user, pass);

                res.send({
                    "route": "/login",
                    "resCode": 0,
                    "signedToken": jwt.signedToken,
                    "comment": `Password for user '${ user }' true.`
                })

                add_token(jwt.signedToken, user);

            }
            else {
                res.send({
                    "route": "/login",
                    "resCode": 1,
                    "comment": `Password for user '${ user }' false.`
                })
            }
        }
        else {
            res.send({
                "route": "/login",
                "resCode": 2,
                "comment": `Username '${ user }' don't exist.`
            })
        }
    }
}

// resCode : [ 0: Token is valid, 1: Token is not valid, 3: Empty token ]
app.get('/testmytoken', (req, res) => {

    let token = req.query['token'];

    if (!token) {
        res.send({
            "resCode": 3,
            "comment": "Please enter a token."
        })
    }
    
    if (verify_token(token)){
        res.send({
            "resCode": 0,
            "comment": "Your token is valid."
        })
    }
    else {
        res.send({
            "resCode": 1,
            "comment": "Your token is not valid."
        })
    }

})

// resCode : [ 0: Logout ok, 1: Already logged out, 3: Empty token ]
app.get('/logout/', (req, res) => {

    let token = req.query['token'];

    if (!token) {
        res.send({
            "resCode": 3,
            "comment": "Please enter a token."
        })
    }

    else {
        if (verify_token(token)){
            remove_token(token);
            res.send({
                "resCode": 0,
                "comment": "Your are now logged out."
            })
        }
        else {
            res.send({
                "resCode": 1,
                "comment": "Your are already logged out."
            })
        }
    }
    

})

/**
 * Request register
 * param : pseudo
 * param : password
 */
app.get('/register/', (req, res) => {
    sql.addUser(req.query['login'], req.query['pass']);
    res.send(`Request for a register (${req.query['login']}, ${req.query['pass']})`);
})

/**
 * Request to share a file with a user
 * param : file_id
 * param : user1
 * param : user2
 * user1 share a file_id with user2
 */
app.get('/share/:file_id/:to_user', (req, res) => {

    let token = req.query["token"]
    let user = verify_token(token);
    console.log("user : " + user)

    if (req.params['to_user'] && req.params['file_id']){
        let to_user = req.params['to_user'];
        let file_id = req.params['file_id'];
        sql.addSharing(user, to_user, file_id).then(function (r) {
            res.send(r);
        })
    }
    else{
        res.send("Unable to share. Please provide a user to share with and a file_id.");
    }
})

/**
 * Request to upload a file
 */
// app.get('/upload/', (req, res) => {
//     let d = new Date();
//     let date_upload = d.getTime();
//     // "abcd", "un", "/a", 46.2054, 6.1459, date_upload



//     res.send(`Request for an upload (${req.query['file']})`)
// });



app.post('/upload*', upload.single('my_file'), (req, res) => {
    // res.send(`Request for an upload (${req.query['file']})`)

    let token = req.query["token"]
    let user = verify_token(token)

    if (user){
        let path = req.params['0']

        if (req.file){
    
            var file_id = Math.round(Math.random() * 100) + Date.now();
            var file_name = "test"
    
    
              
    
            let lat;
            let lon;
    
            http.get('http://bot.whatismyipaddress.com', (result) => {
                result.setEncoding('utf8');
                result.on('data', function(chunk){
                    let ip = chunk;
                    http.get('http://ip-api.com/json/' + ip, (resp) => {
                        let data = '';
                        resp.on('data', (chunk) => {
                            data += chunk;
                        });
        
                        resp.on('end', () => {
                            lat = JSON.parse(data).lat.toFixed(3);
                            lon = JSON.parse(data).lon.toFixed(3);
    
                            // add to BDD
                            sql.addFile(file_id, file_name, user, path, lat, lon, (result) => {
                                res.send(result)
                            })
    
                        });
                    });
                });
            });
        }
        else{
            console.log("Please enter a file");
        }
    }
    else{
        res.send("Please be connected")
    }
    

})

app.get('/download/:file_id', (req, res) => {
    let token = req.query["token"]
    

    let user = verify_token(token);
    if (user){
        sql.verifyFileID(user, file_id, (filename) => {
            console.log(filename)
            if (filename){
                res.download("files/" + file_id, filename);
            }
            else{
                res.send("Can't download (not your file).")
            }
        })
    }
    else{
        res.send("Can't download (please be connected).")
    }
})

/**
 * This function return the content of a path
 * param path : Path of the folder we want to show
 */
app.get('/change-path*', (req, res) => {
    let path = req.params['0'].split("/");
    path.pop();
    path = path.join("/");
    let tok = req.params['0'].split("/").pop();
    let name = verify_token(tok);
    if (name) {
        sql.changeDirectory(name, path,
        (content) => {
            res.send(content);
        });
    }
})

app.get('/create-path*', (req, res) => {
    let c = req.params['0'];

    let tok = c.split("/").pop();
    let path = c.split("/");
    path.pop();
    path = path.join("/");

    let parent = path.split("/");
    parent.pop();
    parent = parent.join("/");

    let name = verify_token(tok);
    if (name) {
        sql.addPath(path, name, parent, (resp, msg) => {
            console.log(resp, msg);
            res.send(resp + " : " + msg);
        });
    }
})

/**
 * Get info of a file
 * @param {string} file_id Id of the file we want the infos
 * @param {string} token Token of the client
 */
app.get('/get-info', (req, res) => {
    let token = req.query["token"];
    let file_id = req.query["file_id"];
    let name = verify_token(token);
    console.log(token);
    if (name){
        sql.getInfo(file_id, name, (resp, msg) => {
            res.send(resp);
        });
    }
});

/**
 * 
 */
app.get('/show-shared-file', (req, res) => {
    let token = req.query["token"];
    let name = verify_token(token);
    if (name){
        sql.getSharedFile(name, (result) => {
            if (result){
                let content = [];
                for (let i in result){
                    console.log(result[i]);
                    content.push({
                        login: result[i].pseudo_2,
                        file_id: result[i].file_id,
                        file_name: result[i].file_name
                    });
                }
                res.send(content);
            }
        });
    }
});

app.use(express.static('front'));
app.use(fileupload());
var upload = multer({ dest: "files/" })
app.listen(port, () => console.log(`Hyperdrive listening on port ${port}!`))
